library(tidyverse)
library(reshape2)

setwd("../191016.nfkb2.censtad.mouse/")

pal <- c("wt" = "#F8766D", "ndg" = "#B79F00", "ngg" = "#00BA38", "lym" = "#00BFC4", "xdr" = "#619CFF", "aire" = "#F564E3")
COI <- c("a", "b")
GOI <- c("wt", "ndg", "ngg", "lym", "xdr", "aire")
SOI <- c("t_1", "t_4", "t_8", "s_r", "s_4", "s_8")
# read in a clonotype table, 
# e.g. Monash University Data Repository, aka "Bridges", DOI: 10.26180/5da7d4b26478f 
df <- read.delim("NFKB2_mouse_TCRseq.txt", header = TRUE) 
# exclude non-functional clonotypes and clonotypes encountered only once
min_count <- 2
df$nf <- str_count(df$cdr3aa, "[^[:upper:]]")
dfx <- df %>% 
  filter(count >= min_count, nf == 0)
###################################################################
# check central CDR3 cysteine usage
dfx$cdr3_trimmed <- as.character(dfx$cdr3aa)
dfx$len <- as.integer(nchar(dfx$cdr3_trimmed))
dfs <- filter(dfx, len > 7)
dfs["n2"] <- as.character("-")
dfs["n1"] <- as.character("-")
dfs["top"] <- as.character("-")
dfs["c1"] <- as.character("-")
dfs["c2"] <- as.character("-")
dfs["top"] <- ifelse(dfs$len>0,(substr(dfs$cdr3_trimmed, floor((nchar(dfs$cdr3_trimmed)/2)+1),floor((nchar(dfs$cdr3_trimmed)/2)+1))),"-")
dfs["c1"] <- ifelse(dfs$len>2,(substr(dfs$cdr3_trimmed, floor((nchar(dfs$cdr3_trimmed)/2)+2),floor((nchar(dfs$cdr3_trimmed)/2)+2))),"-")
dfs["n1"] <- ifelse(dfs$len>1,(substr(dfs$cdr3_trimmed, floor((nchar(dfs$cdr3_trimmed)/2)-0),floor((nchar(dfs$cdr3_trimmed)/2)-0))),"-")
dfs["c2"] <- ifelse(dfs$len>4,(substr(dfs$cdr3_trimmed, floor((nchar(dfs$cdr3_trimmed)/2)+3),floor((nchar(dfs$cdr3_trimmed)/2)+3))),"-")
dfs["n2"] <- ifelse(dfs$len>3,(substr(dfs$cdr3_trimmed, floor((nchar(dfs$cdr3_trimmed)/2)-1),floor((nchar(dfs$cdr3_trimmed)/2)-1))),"-")
dfs$cencys <- dfs$n2 == "C" | dfs$n1 == "C" | dfs$top == "C" |dfs$c1 == "C" | dfs$c2 == "C"
###################################################################
df1 <- dfs %>%
  dcast(genotype + mouse + chain + subset ~ cencys, value.var = 'count', length) %>%
  mutate(
    sequences = `FALSE` + `TRUE`,
    freq = `TRUE` / sequences
  )
# for samples with 0 sequences with Cys in the CDR3 apex, the cysteine index is undefined
# "freq_corr" is a proxy cysteine index, equal to the reciprocal of the number of unique sequences in the sample
df1$freq_corr <- df1$freq + (df1$`TRUE`==0)/(df1$sequences)  
df1$log_freq <- log10(df1$freq_corr)
# "lod" stands for "limit of detection", a column that identifies samples for which the cysteine index is undefined
df1$lod <- df1$`TRUE` == 0
###################################################################
# check for Stadinski's (mostly hydrophobic) doublets that promote T cell self-reactivity
dfs["p6"] <- substr(dfs$cdr3aa, 6, 6)
dfs["p7"] <- substr(dfs$cdr3aa, 7, 7)
stad <- read.delim("../stadinski.txt", header = T)
stad_S <- filter(stad, reactivity == "S")
dfs$p67 <- paste0(dfs$p6, dfs$p7)
# create a workaround for doublets wrongly designated as 'NA' for 'not available'
dfs$p67[dfs$p67 == "NA"] <- "NX"
dfs$S_d <- dfs$p67 %in% stad_S$doub 
df4 <- dfs %>%
  dcast(genotype + mouse + chain + subset ~ S_d, value.var = "count", length) %>%
  mutate(
    sequences = `FALSE` + `TRUE`,
    freq_S = `TRUE` / sequences
  ) 

# prepare a dataframe to summarize the TCRseq results by sample

names(df1)[5:6] <- c("Cys_F", "Cys_T")
names(df4)[5:6] <- c("P6_P7_F", "P6_P7_T")
dfo <- merge(df1, df4, by = c("genotype", "mouse", "subset", "chain")) 
# for plotting, designate the desired levels for factor vectors in the dataframe 
dfo$genotype <- factor(dfo$genotype, levels = GOI)
dfo$subset <- factor(dfo$subset, levels = SOI)
dfo <- arrange(dfo,  subset, chain, genotype, mouse)
dfo$xanc <- match(dfo$genotype, GOI)
shi <- c(-.2, .2)
dfo$x <- dfo$xanc + shi[dfo$chain]

write.table(dfo, file =   paste("censtad.nfkb2", COI[1],COI[2],
                                "subs", length(SOI), min_count, "txt", sep = "."),
            sep = "\t", row.names = F, quote = F)

pdf(file = paste(DOA, "stad.nfkb2", COI[1],COI[2],
                 "subs", length(SOI), min_count, "pdf", sep = "."), 
    height = 3, width = 7)
g <- ggplot(dfo , 
            aes(x=x, y = freq_S, shape = chain, color = genotype)) +
  geom_jitter(width = .07) + facet_grid( ~ subset) +
  scale_shape_manual(values = c(22,21)) +
  scale_color_manual(values = pal) +
  theme_bw() +
  scale_y_continuous(limits = c(0, .3),
                     breaks = c(0,.1,.2,.3),
                     labels = c("0", "10", "20", "30")) + 
  scale_x_continuous(breaks = seq(1,length(GOI)),
                     labels = GOI) +
  labs(y = "% clonotypes with a\nself-reactive CDR3 P6-P7 doublet") +
  theme(panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        panel.grid.minor.y = element_blank(),
        axis.text.x = element_text(angle = 45, hjust = 1, vjust = 1),
        legend.position = 'none',
        axis.title.x = element_blank()
  ) 
g                     
dev.off()

pdf(file = paste(DOA, "stad.nfkb2", COI[1],COI[2],
                 "subs", length(SOI), min_count, "clean.pdf", sep = "."), 
    height = 1.2, width = 5)
g <- ggplot(dfo , 
            aes(x=x, y = freq_S, shape = chain, color = genotype)) +
  geom_jitter(width = .1, size = .7, stroke = .3) + facet_grid( ~ subset) +
  scale_shape_manual(values = c(22,21)) +
  scale_color_manual(values = pal) +
  theme_bw() +
  scale_y_continuous(limits = c(0, .3),
                     breaks = c(0,.1,.2,.3),
                     labels = c("0", "10", "20", "30")) + 
  scale_x_continuous(breaks = seq(1,length(GOI)),
                     labels = GOI) +
  theme(panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        panel.grid.minor.y = element_blank(),
        axis.text = element_blank(),
        legend.position = 'none',
        axis.title = element_blank(),
        strip.background = element_blank(),
        strip.text = element_blank(),
        axis.ticks.x = element_blank()
  ) 
g                     
dev.off()

pdf(file = paste(DOA, "cys.nfkb2", COI[1],COI[2],
                 "subs", length(SOI), min_count, "pdf", sep = "."), 
    height = 3, width = 7)
g <- ggplot(dfo , 
            aes(x=x, y = log_freq, shape = chain, color = genotype, fill = lod)) +
  geom_jitter(width = .07) + facet_grid( ~ subset) +
  scale_shape_manual(values = c(22,21)) +
  scale_fill_manual(values = c("white", "gray50")) +
  scale_color_manual(values = pal) +
  theme_bw() +
  scale_y_continuous(limits = c(-4, -1),
                     breaks = seq(-4,-1),
                     labels = c("0.01", "0.1", "1", "10")) + 
  scale_x_continuous(breaks = seq(1,length(GOI)),
                     labels = GOI) +
  labs(y = "% clonotypes with Cys within\n2 positions of CDR3 apex") +
  theme(panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        panel.grid.minor.y = element_blank(),
        axis.text.x = element_text(angle = 45, hjust = 1, vjust = 1),
        legend.position = 'none',
        axis.title.x = element_blank()
  ) 
g                     
dev.off()

pdf(file = paste(DOA, "cys.nfkb2", COI[1],COI[2],
                 "subs", length(SOI), min_count, "clean.pdf", sep = "."), 
    height = 1.2, width = 5)
g <- ggplot(dfo , 
            aes(x=x, y = log_freq, shape = chain, color = genotype, fill = lod)) +
  geom_jitter(width = .1, height = 0, size = .7, stroke = .3) + facet_grid( ~ subset) +
  scale_shape_manual(values = c(22,21)) +
  scale_fill_manual(values = c("white", "gray50")) +
  scale_color_manual(values = pal) +
  theme_bw() +
  scale_y_continuous(limits = c(-4, -1),
                     breaks = seq(-4,-1),
                     labels = c("0.01", "0.1", "1", "10")) + 
  scale_x_continuous(breaks = seq(1,length(GOI)),
                     labels = GOI) +
  theme(panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        panel.grid.minor.y = element_blank(),
        axis.text = element_blank(),
        legend.position = 'none',
        axis.title = element_blank(),
        strip.background = element_blank(),
        strip.text = element_blank(),
        axis.ticks.x = element_blank()
  ) 
g                     
dev.off()

